[GtkTemplate(ui = "/io/gitlab/nekocwd/pipetap/gui/slider_overlay.ui")]
public class PipeTap.Ui.SliderOverlay : PipeTap.Ui.FloatingWindow {
    [GtkChild]
    private unowned Gtk.Scale scale;
    [GtkChild]
    private unowned Gtk.ToggleButton toggle_btn;
    [GtkChild]
    private unowned Gtk.Button press_btn;

    public Logic.Ctrl.CtrlType slider_type { get; set; }
    public Logic.Ctrl.CtrlType? toggle_type { get; set; }
    public bool is_available { get; protected set; }
    public bool is_toggle { get; protected set; }
    public string icon { get; set; }
    public bool is_btn_available { get; protected set; }

    public SliderOverlay(PipeTap.Application app, Logic.Ctrl.CtrlType slider_type, Logic.Ctrl.CtrlType toggle_type, string name, string icon_name, bool is_toggle = true) {
        base(app, name);
        this.slider_type = slider_type;
        this.toggle_type = toggle_type;
        this.is_toggle = is_toggle;
        scale.value_changed.connect(value_changed);
        toggle_btn.toggled.connect(lock_changed);
        press_btn.clicked.connect(lock_changed);
        app.notify["device"].connect(set_ctrls);
        this.icon = icon_name;
        set_ctrls();
    }

    void set_ctrls() {
        var slider_ctrl = Logic.find_ctrl(slider_type);
        var lock_ctrl = Logic.find_ctrl(toggle_type);
        scale.sensitive = slider_ctrl != null;
        is_btn_available = lock_ctrl != null;
        is_available = slider_ctrl != null || lock_ctrl != null;
        if (slider_ctrl != null)
            scale.set_value(slider_ctrl.value);
        if (lock_ctrl != null)
            toggle_btn.active = lock_ctrl.value == 1;
    }

    void value_changed() {
        message("Value %f", scale.get_value());
        var ctrl = Logic.find_ctrl(slider_type);
        if (ctrl == null) {
            message("Slider ctrl not found");
            return;
        }
        ctrl.value = (int) scale.get_value();
    }

    void lock_changed() {
        var ctrl = Logic.find_ctrl(toggle_type);
        if (ctrl == null) {
            message("Toggle ctrl not found");
            return;
        }
        if (!is_toggle) {
            ctrl.value = 1;
            return;
        }
        ctrl.value = (int) toggle_btn.active;
    }
}
